"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMessageForSlack = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const axios_1 = require("axios");
const winston_slack_webhook_transport_1 = __importDefault(require("winston-slack-webhook-transport"));
const getMessageForSlack = ({ info, nodeEnv, appName, levels }) => {
    if (typeof info !== 'object')
        return false;
    const errorObject = {
        context: info.context ?? undefined,
        message: info?.message ?? undefined,
        level: info.level ?? undefined,
        data: info.data ?? undefined,
        timestamp: info.timestamp ?? undefined,
    };
    const { data, context, message } = errorObject;
    let { level } = errorObject;
    if (level === undefined)
        return false;
    level = level.toLowerCase();
    /**
     * if send is disabled for received level, return
     */
    if (levels[level].send === false)
        return false;
    let errorMessage = message;
    const attachments = [];
    const fields = [];
    if (data !== undefined) {
        if (data instanceof Error || data instanceof axios_1.AxiosError) {
            errorMessage = data.message;
            if (data.stack !== undefined) {
                const stackParts = data.stack.split('\n');
                stackParts.forEach((part) => fields.push({
                    title: '',
                    value: part.trim(),
                    short: false,
                }));
            }
        }
    }
    attachments.push({
        // green
        color: '#36a64f',
        fields,
        footer: `*${errorObject.timestamp} - ${nodeEnv}*`.toUpperCase(),
    });
    const slackMessage = {
        attachments,
        blocks: [
            /**
             * Header
             */
            {
                type: 'header',
                text: {
                    type: 'plain_text',
                    text: `${errorObject.level.toUpperCase()}:exclamation:`,
                    emoji: true,
                },
            },
            /**
             * context
             */
            {
                type: 'rich_text',
                elements: [
                    {
                        type: 'rich_text_section',
                        elements: [
                            {
                                type: 'text',
                                text: 'Error in: ',
                            },
                            {
                                type: 'text',
                                text: appName,
                                style: {
                                    bold: true,
                                },
                            },
                        ],
                    },
                ],
            },
            {
                type: 'divider',
            },
            /**
             * Context  Message
             */
            {
                type: 'section',
                fields: [
                    {
                        type: 'mrkdwn',
                        text: '*Message:*',
                    },
                    {
                        type: 'mrkdwn',
                        text: message,
                    },
                ],
            },
            {
                type: 'divider',
            },
            /**
             * Context
             */
            {
                type: 'section',
                fields: [
                    {
                        type: 'mrkdwn',
                        text: '*Context:*',
                    },
                    {
                        type: 'mrkdwn',
                        text: context,
                    },
                ],
            },
            {
                type: 'divider',
            },
            /**
             * Error Stack or Response
             */
            {
                type: 'section',
                fields: [
                    {
                        type: 'mrkdwn',
                        text: '*Error:*',
                    },
                    {
                        type: 'mrkdwn',
                        text: errorMessage,
                    },
                ],
            },
        ],
    };
    return slackMessage;
};
exports.getMessageForSlack = getMessageForSlack;
class SlackTransport {
    static create({ appName, webhookUrl, nodeEnv, levels }) {
        return new winston_slack_webhook_transport_1.default({
            webhookUrl,
            formatter: (info) => (0, exports.getMessageForSlack)({ appName, info, nodeEnv, levels }),
        });
    }
}
exports.default = SlackTransport;
