/* eslint-disable @typescript-eslint/no-explicit-any */
import { AxiosError } from 'axios';
import SlackHook, { SlackMessage } from 'winston-slack-webhook-transport';

import { LOG_LEVEL } from '#/logger/logger.types';

import { AttachementField, Attachment, GetMessageForSlack, SlackCreateParams } from './slack.transport.types';

export const getMessageForSlack = ({ info, nodeEnv, appName, levels }: GetMessageForSlack): false | SlackMessage => {
	if (typeof info !== 'object') return false;

	const errorObject = {
		context: info.context ?? undefined,
		message: info?.message ?? undefined,
		level: (info.level as LOG_LEVEL as LOG_LEVEL) ?? undefined,
		data: info.data ?? undefined,
		timestamp: (info.timestamp as string) ?? undefined,
	};

	const { data, context, message } = errorObject;
	let { level } = errorObject;

	if (level === undefined) return false;

	level = level.toLowerCase() as LOG_LEVEL;

	/**
	 * if send is disabled for received level, return
	 */
	if (levels[level].send === false) return false;

	let errorMessage = message;

	const attachments: Attachment[] = [];
	const fields: AttachementField[] = [];

	if (data !== undefined) {
		if (data instanceof Error || data instanceof AxiosError) {
			errorMessage = data.message;
			if (data.stack !== undefined) {
				const stackParts = data.stack.split('\n');

				stackParts.forEach((part) =>
					fields.push({
						title: '',
						value: part.trim(),
						short: false,
					})
				);
			}
		}
	}

	attachments.push({
		// green
		color: '#36a64f',
		fields,
		footer: `*${errorObject.timestamp} - ${nodeEnv}*`.toUpperCase(),
	});

	const slackMessage: SlackMessage = {
		attachments,
		blocks: [
			/**
			 * Header
			 */
			{
				type: 'header',
				text: {
					type: 'plain_text',
					text: `${errorObject.level.toUpperCase()}:exclamation:`,
					emoji: true,
				},
			},

			/**
			 * context
			 */
			{
				type: 'rich_text',
				elements: [
					{
						type: 'rich_text_section',
						elements: [
							{
								type: 'text',
								text: 'Error in: ',
							},
							{
								type: 'text',
								text: appName,
								style: {
									bold: true,
								},
							},
						],
					},
				],
			},
			{
				type: 'divider',
			},
			/**
			 * Context  Message
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Message:*',
					},
					{
						type: 'mrkdwn',
						text: message,
					},
				],
			},
			{
				type: 'divider',
			},
			/**
			 * Context
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Context:*',
					},
					{
						type: 'mrkdwn',
						text: context,
					},
				],
			},
			{
				type: 'divider',
			},

			/**
			 * Error Stack or Response
			 */
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error:*',
					},
					{
						type: 'mrkdwn',
						text: errorMessage,
					},
				],
			},
		],
	};
	return slackMessage;
};

export default class SlackTransport {
	public static create({ appName, webhookUrl, nodeEnv, levels }: SlackCreateParams) {
		return new SlackHook({
			webhookUrl,
			formatter: (info: any) => getMessageForSlack({ appName, info, nodeEnv, levels }),
		});
	}
}
