import { Inject, Injectable } from '@nestjs/common';
import * as winston from 'winston';

import { LOG_LEVEL, Logger } from '#/logger/logger.types';

export const WinstonLoggerTransportsKey = Symbol();

@Injectable()
export default class WinstonLogger implements Logger {
	private logger: winston.Logger;

	public constructor(@Inject(WinstonLoggerTransportsKey) transports: winston.transport[]) {
		// Create winston logger
		this.logger = winston.createLogger(this.getLoggerFormatOptions(transports));
	}

	private getLoggerFormatOptions(transports: winston.transport[]) {
		// Setting log levels for winston
		// eslint-disable-next-line @typescript-eslint/no-explicit-any
		const levels: any = {};
		let cont = 0;
		Object.values(LOG_LEVEL).forEach((level) => {
			levels[level] = cont;
			cont++;
		});

		return {
			level: LOG_LEVEL.Debug,
			levels,
			format: winston.format.combine(
				// Add timestamp and format the date
				winston.format.timestamp({
					format: 'DD/MM/YYYY, HH:mm:ss',
				}),
				// Errors will be logged with stack trace
				winston.format.errors({ stack: true }),
				// Add custom Log fields to the log
				winston.format((info) => {
					// Info contains an Error property

					// console.log('--- WINSTONM LOOGER', info);
					if (info.data && info.data instanceof Error) {
						info.stack = info.data.stack;
						info.data = undefined;
					}

					let splat = info[Symbol.for('splat')] ?? undefined;

					if (splat !== undefined) {
						if (Array.isArray(splat)) {
							splat = splat[0];
						}

						info.context = splat?.context ?? undefined;
						info.data = splat?.data ?? undefined;
					}

					// console.log(info);
					return info;
				})(),
				winston.format.json()
			),
			transports: transports,
			exceptionHandlers: transports,
			rejectionHandlers: transports,
		};
	}

	public log(level: LOG_LEVEL, message: string | Error, context?: string, data?: unknown) {
		const m = message instanceof Error ? message.message : message;

		const payload = {
			context,
			data,
		};

		this.logger.log(level, m, payload);
	}

	public debug(message: string, context: string, data?: unknown) {
		this.log(LOG_LEVEL.Debug, message, context, data);
	}

	public info(message: string, context: string, data?: unknown) {
		this.log(LOG_LEVEL.Info, message, context, data);
	}

	public warn(message: string | Error, context: string, data?: unknown) {
		this.log(LOG_LEVEL.Warn, message, context, data);
	}

	public error(message: string | Error, context: string, data?: unknown) {
		this.log(LOG_LEVEL.Error, message, context, data);
	}

	public fatal(message: string | Error, context: string, data?: unknown) {
		this.log(LOG_LEVEL.Fatal, message, context, data);
	}

	public emergency(message: string | Error, context: string, data?: unknown) {
		this.log(LOG_LEVEL.Emergency, message, context, data);
	}
}
