import { trace } from '@opentelemetry/api';
import { diag, DiagConsoleLogger, DiagLogLevel } from '@opentelemetry/api';
import { LogRecord, logs, SeverityNumber } from '@opentelemetry/api-logs';
import { getNodeAutoInstrumentations } from '@opentelemetry/auto-instrumentations-node';
import { OTLPLogExporter } from '@opentelemetry/exporter-logs-otlp-http';
import { OTLPMetricExporter } from '@opentelemetry/exporter-metrics-otlp-http';
import { OTLPTraceExporter } from '@opentelemetry/exporter-trace-otlp-http';
import { Resource } from '@opentelemetry/resources';
import { BatchLogRecordProcessor } from '@opentelemetry/sdk-logs';
import { PeriodicExportingMetricReader } from '@opentelemetry/sdk-metrics';
import { NodeSDK } from '@opentelemetry/sdk-node';
import { BatchSpanProcessor } from '@opentelemetry/sdk-trace-base';
import { SemanticResourceAttributes } from '@opentelemetry/semantic-conventions';
import { config } from 'dotenv';
import express, { Response } from 'express';

config();

// For troubleshooting, set the log level to DiagLogLevel.DEBUG
const setDebugger = (LogLevel: DiagLogLevel) => diag.setLogger(new DiagConsoleLogger(), LogLevel);

const env = {
	nodeEnv: process.env.NODE_ENV || 'local',
	service: {
		name: process.env.SERVER_HOSTNAME || 'undefined service',
		version: process.env.npm_package_version || 'undefined service version',
	},
	collector: {
		url: process.env.OPENTELEMETRY_COLLECTOR_URL || 'http://otelcol:4318',
	},
};

const resource = new Resource({
	[SemanticResourceAttributes.SERVICE_NAME]: env.service.name,
	[SemanticResourceAttributes.SERVICE_VERSION]: env.service.version,
});

const traceExporter = new OTLPTraceExporter({
	url: `${env.collector.url}/v1/traces`,
});

const spanProcessor = new BatchSpanProcessor(traceExporter);

const metricReader =
	env.nodeEnv !== 'local'
		? new PeriodicExportingMetricReader({
				exporter: new OTLPMetricExporter({
					url: `${env.collector.url}/v1/metrics`,
				}),
				exportIntervalMillis: 10000,
		  })
		: undefined;

const logRecordProcessor =
	env.nodeEnv! === 'local'
		? new BatchLogRecordProcessor(
				new OTLPLogExporter({
					url: `${env.collector.url}/v1/logs`,
				})
		  )
		: undefined;

const otelSdk = new NodeSDK({
	resource,
	spanProcessor,
	metricReader,
	logRecordProcessor,
	instrumentations: [
		getNodeAutoInstrumentations({
			'@opentelemetry/instrumentation-fs': {
				enabled: false,
			},
		}),
	],
});

// gracefully shut down the SDK on process exit
process.on('SIGTERM', () => {
	otelSdk
		.shutdown()
		.then(
			() => console.log('Open Telemtry SDK shut down successfully'),
			(err) => console.log('Error shutting down Open Telemtry  SDK', err)
		)
		.finally(() => process.exit(0));
});

const app = express();
/**
 * Open Telemtry Logging middleware
 */

const loggingMiddleware = app.use((req, res, next) => {
	try {
		const activeSpan = trace.getActiveSpan();
		const logger = logs.getLogger(env.service.name);

		if (activeSpan !== undefined && logger !== undefined) {
			const temp = res.json;

			interface emitParams {
				severityNumber: SeverityNumber;
				severityText: string;
				context: string;
				message: unknown;
			}

			const log = ({ severityNumber, severityText, context, message }: emitParams) => {
				const obj: LogRecord = {
					severityNumber,
					severityText,
					body: JSON.stringify({ context, message }),
				};
				console.log('--- LOG ---', obj);
				logger.emit(obj);
			};

			res.json = function (...args): Response<unknown, Record<string, unknown>> {
				log({
					severityNumber: SeverityNumber.INFO,
					severityText: 'INFO',
					context: 'express request log',
					message: {
						request: {
							headers: req.headers,
							body: req.body,
							query: req.query,
							params: req.params,
						},
						response: {
							body: args['0'],
						},
					},
				});

				return temp.apply(this, args);
			};
		}
	} catch (e) {
		console.log('--- ERR :( ---', e);
	}
	next();
});

const logger = logs.getLogger(env.service.name);

export { DiagLogLevel, logger, loggingMiddleware, otelSdk, setDebugger, trace };
